from forbiddenfruit import curse as monkey_patch
from omakase.functions import *


monkey_patch(list, 'len', lambda sequence: len(sequence))
monkey_patch(list, 'map', lambda sequence, function: map(function, sequence))
monkey_patch(list, 'filter', lambda sequence, function: filter(function, sequence))
monkey_patch(list, 'max', lambda sequence: max(sequence) if sequence else None)
monkey_patch(list, 'min', lambda sequence: min(sequence) if sequence else None)
monkey_patch(list, 'sum', lambda sequence: sum(sequence))
monkey_patch(list, 'zip', lambda sequence, other: zip(sequence, other))


monkey_patch(list, 'join', join)
monkey_patch(list, 'sortby', sortby)
monkey_patch(list, 'compact', compact)
monkey_patch(list, 'freq', freq)
monkey_patch(list, 'groupby', groupby)
monkey_patch(list, 'indexby', indexby)
monkey_patch(list, 'uniq', uniq)
monkey_patch(list, 'take', take)
monkey_patch(list, 'drop', drop)
monkey_patch(list, 'first', first)
monkey_patch(list, 'last', last)
monkey_patch(list, 'reduce', my_reduce)



# Functions that I am missing from Ruby


def join(sequence, separator):
    return separator.join(sequence)


def sortby(sequence, function):
    return sorted(sequence, key=function)


def compact(sequence):
    result = []
    for each in sequence:
        if each is None: continue
        result.append(each)

    return result


def freq(sequence, function=None):
    mapping = {}
    for each in sequence:
        key = function(each) if function else each
        mapping[key] = mapping.get(key, 0) + 1

    return mapping


def groupby(sequence, function=None):
    mapping = {}
    for each in sequence:
        key = function(each) if function else each
        if not key in mapping: mapping[key] = []
        mapping[key].append(each)

    return mapping


def indexby(sequence, function=None):
    mapping = {}
    for each in sequence:
        key = function(each) if function else each
        if key in mapping: continue
        mapping[key] = each

    return mapping


def uniq(sequence):
    reply = []
    mapping = {}
    for each in sequence:
        if each in mapping: continue
        mapping[each] = True
        reply.append(each)

    return reply


def take(sequence, n):
    return sequence[:n]


def drop(sequence, n):
    return sequence[n:]


def first(sequence):
    return sequence[0] if sequence else None


def last(sequence):
    return sequence[-1] if sequence else None


def my_reduce(sequence, function, initial=None):
    if initial:
        return reduce(function, sequence, initial)
    elif not sequence:
        return None
    else:
        return reduce(function, sequence)



from distutils.core import setup

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except:
    long_description = None


setup(
  name = 'omakase',
  packages = ['omakase'],
  version = '0.6.2',
  description = 'My personal functions.',
  long_description = long_description,
  author = 'Adrian Kuhn',
  author_email = 'akuhnplus@gmail.com',
  url = 'https://github.com/akuhn/py-omakase',
  install_requires = ['forbiddenfruit'],
)
